// Depth buffer copied from other shaders to this input with 3Dmigoto:
Texture2D<float4> DepthBuffer : register(t110);

cbuffer ConstantShared : register(b13)
{
  
  float4 NEARFAR : packoffset(c13);
  float4 VIEWPORTOFFSET : packoffset(c14);
  uint4 cLightCount : packoffset(c15);
  uint4 cProbeCount : packoffset(c16);
  float4 cProbePositionAndRadius[32] : packoffset(c17);
  float4 cProbeScaleAndType[32] : packoffset(c49);
  float4 cProbeOrientation[32] : packoffset(c81);
  uint4 cProbeCubeIndex[8] : packoffset(c113);
  float4 cProbeInnerInfluence[8] : packoffset(c121);
  float4x4 CSMMATRIX[4] : packoffset(c129);
  float4 CSMRANGE : packoffset(c145);
  float4 CSMBIAS : packoffset(c146);
  float4 tCascadeAtlasSIZE : packoffset(c147);
  float4 tRotationVectorSIZE : packoffset(c148);
  float4 tStaticShadowMapSIZE : packoffset(c149);
  float4 cSpotAndPointSettings[5] : packoffset(c150);
  float4x4 cMatPointShadowViewProj[24] : packoffset(c155);
  float4x4 cMatSpotShadowViewProj[5] : packoffset(c251);
  float4x4 cMatStaticShadowViewProj : packoffset(c271);
  float4 cVecInvPixelSize : packoffset(c275);
  uint4 cRenderMode : packoffset(c276);
}


float world_z_from_depth_buffer(float x, float y)
{
	uint width, height;
	float z;
		
	DepthBuffer.GetDimensions(width, height);

	x = min(max((x / 2 + 0.5) * width, 0), width - 1);
	y = min(max((-y / 2 + 0.5) * height, 0), height - 1);
	z = DepthBuffer.Load(int3(x, y, 0)).x;
	
	if (z == 1)
        return 0;
	  
  
  float4 r0;
  r0.w = -NEARFAR.x + z;
  r0.w = min(-9.99999994e-009, r0.w);
  return NEARFAR.y / r0.w;
	
	
}

float adjust_from_depth_buffer(float x, float y)
{
	float4 stereo = StereoParams.Load(0);
	float separation = stereo.x; float convergence = stereo.y;
	float old_offset, offset, w, sampled_w, distance;
	uint i;

	// Stereo cursor: To improve the accuracy of the stereo cursor, we
	// sample a number of points on the depth buffer, starting at the near
	// clipping plane and working towards original x + separation.
	//
	// You can think of this as a line in three dimensional space that
	// starts at each eye and stretches out towards infinity. We sample 255
	// points along this line (evenly spaced in the X axis) and compare
	// with the depth buffer to find where the line is first intersected.
	//
	// Note: The reason for sampling 255 points came from a restriction in
	// DX9/SM3 where loops had to run a constant number of iterations and
	// there was no way to set that number from within the shader itself.
	// I'm not sure if the same restriction applies in DX11 with SM4/5 - if
	// it doesn't, we could change this to check each pixel instead for
	// better accuracy.
	

	offset = -convergence * separation;	// Z = X offset from center
	distance = separation - offset;	// Total distance to cover (separation - starting X offset)

	old_offset = offset;
	for (i = 0; i < 255; i++) {
		offset += distance / 255.0;

		// Calculate depth for this point on the line:
		w = (separation * convergence) / (separation - offset);

		sampled_w = world_z_from_depth_buffer(x + offset, y);
		if (sampled_w == 0)
			return 0;

		// If the sampled depth is closer than the calculated depth,
		// we have found something that intersects the line, so exit
		// the loop and return the last point that was not intersected:
		if (w > sampled_w)
			break;

		old_offset = offset;
	}

	return old_offset;
}
