//Inverse

cbuffer ConstantShared : register(b0)
{
  float4x4 VIEW : packoffset(c0);
  float4x4 PROJECTION : packoffset(c4);
  float4x4 VIEWPROJ : packoffset(c8);
  float4 CAMPOS : packoffset(c12);
  float4 NEARFAR : packoffset(c13);
  float4 VIEWPORTOFFSET : packoffset(c14);
  uint4 cLightCount : packoffset(c15);
  uint4 cProbeCount : packoffset(c16);
  float4 cProbePositionAndRadius[32] : packoffset(c17);
  float4 cProbeScaleAndType[32] : packoffset(c49);
  float4 cProbeOrientation[32] : packoffset(c81);
  uint4 cProbeCubeIndex[8] : packoffset(c113);
  float4 cProbeInnerInfluence[8] : packoffset(c121);
  float4x4 CSMMATRIX[4] : packoffset(c129);
  float4 CSMRANGE : packoffset(c145);
  float4 CSMBIAS : packoffset(c146);
  float4 tCascadeAtlasSIZE : packoffset(c147);
  float4 tRotationVectorSIZE : packoffset(c148);
  float4 tStaticShadowMapSIZE : packoffset(c149);
  float4 cSpotAndPointSettings[5] : packoffset(c150);
  float4x4 cMatPointShadowViewProj[24] : packoffset(c155);
  float4x4 cMatSpotShadowViewProj[5] : packoffset(c251);
  float4x4 cMatStaticShadowViewProj : packoffset(c271);
  float4 cVecInvPixelSize : packoffset(c275);
  uint4 cRenderMode : packoffset(c276);
}

RWBuffer<float4> InvViewMatrixOutputBuffer : register(u0);

float4 inverse_transpose_parallel(matrix m, uint pos)
{
	uint3 idx;
	float4 tmp;

	idx = pos < uint3(1, 2, 3) ? uint3(1, 2, 3) : uint3(0, 1, 2);
	float add = pos % 2 == 0 ? 1.0 : -1.0;

	tmp = m[idx.x].yxxx*(add*m[idx.y].zwyz*m[idx.z].wzwy - add*m[idx.y].wzwy*m[idx.z].zwyz)
	    + m[idx.x].zzyy*(add*m[idx.y].wxwx*m[idx.z].ywxz - add*m[idx.y].ywxz*m[idx.z].wxwx)
	    + m[idx.x].wwwz*(add*m[idx.y].yzxy*m[idx.z].zxyx - add*m[idx.y].zxyx*m[idx.z].yzxy);
	return tmp / determinant(m);
}

[numthreads(4, 1, 1)]
void main(uint3 tid: SV_DispatchThreadID)
{
	InvViewMatrixOutputBuffer[tid.x] = inverse_transpose_parallel(VIEWPROJ, tid.x);
}